@@ -29,18 +29,14 @@
 
 
 using System;
+using System.Collections.Generic;
 using System.Net;
 using System.Net.Sockets;
 using System.Xml;
-using System.Web;
-using System.Web.Hosting;
-using System.Collections;
-using System.Collections.Generic;
 using System.Text;
 using System.Threading;
 using System.IO;
-using System.Globalization;
-using System.Runtime.InteropServices;
+using Mono.WebServer.Log;
 
 namespace Mono.WebServer
 {
@@ -84,33 +80,26 @@ namespace Mono.WebServer
 	
 	public class ApplicationServer : MarshalByRefObject
 	{
-		WebSource webSource;
+		readonly WebSource webSource;
 		bool started;
 		bool stop;
-		bool verbose;
 		Socket listen_socket;
-		bool single_app;
-		Exception initialException;
-		string physicalRoot;
-		
+
 		Thread runner;
 
 		// This is much faster than hashtable for typical cases.
-		ArrayList vpathToHost = new ArrayList ();
+		readonly List<VPathToHost> vpathToHost = new List<VPathToHost> ();
 
-		public bool SingleApplication {
-			get { return single_app; }
-			set { single_app = value; }
-		}
+		public bool SingleApplication { get; set; }
 
 		public IApplicationHost AppHost {
-			get { return ((VPathToHost) vpathToHost [0]).AppHost; }
-			set { ((VPathToHost) vpathToHost [0]).AppHost = value; }
+			get { return vpathToHost [0].AppHost; }
+			set { vpathToHost [0].AppHost = value; }
 		}
 
 		public IRequestBroker Broker {
-			get { return ((VPathToHost) vpathToHost [0]).RequestBroker; }
-			set { ((VPathToHost) vpathToHost [0]).RequestBroker = value; }
+			get { return vpathToHost [0].RequestBroker; }
+			set { vpathToHost [0].RequestBroker = value; }
 		}
 
 		public int Port {
@@ -118,7 +107,7 @@ namespace Mono.WebServer
 				if (listen_socket == null || !listen_socket.IsBound)
 					return -1;
 
-				IPEndPoint iep = listen_socket.LocalEndPoint as IPEndPoint;
+				var iep = listen_socket.LocalEndPoint as IPEndPoint;
 				if (iep == null)
 					return -1;
 
@@ -126,9 +115,9 @@ namespace Mono.WebServer
 			}
 		}
 
-		public string PhysicalRoot {
-			get { return physicalRoot; }
-		}
+		public string PhysicalRoot { get; private set; }
+
+		public bool Verbose { get; set; }
 
 		[Obsolete ("Use the .ctor that takes a 'physicalRoot' argument instead")]
 		public ApplicationServer (WebSource source) : this (source, Environment.CurrentDirectory)
@@ -139,16 +128,11 @@ namespace Mono.WebServer
 		{
 			if (source == null)
 				throw new ArgumentNullException ("source");
-			if (physicalRoot == null || physicalRoot.Length == 0)
+			if (String.IsNullOrEmpty (physicalRoot))
 				throw new ArgumentNullException ("physicalRoot");
 			
-			this.webSource = source;
-			this.physicalRoot = physicalRoot;
-		} 
-
-		public bool Verbose {
-			get { return verbose; }
-			set { verbose = value; }
+			webSource = source;
+			PhysicalRoot = physicalRoot;
 		}
 
 		public void AddApplication (string vhost, int vport, string vpath, string fullPath)
@@ -157,15 +141,14 @@ namespace Mono.WebServer
 			if (fullPath != null && !fullPath.EndsWith (dirSepChar.ToString ()))
 				fullPath += dirSepChar;
 			
-			// TODO - check for duplicates, sort, optimize, etc.
-			if (verbose && !single_app) {
-				Console.WriteLine("Registering application:");
-				Console.WriteLine("    Host:          {0}", (vhost != null) ? vhost : "any");
-				Console.WriteLine("    Port:          {0}", (vport != -1) ?
-						  vport.ToString () : "any");
+			// TODO: Check for duplicates, sort, optimize, etc.
+			if (Verbose && !SingleApplication) {
+				Logger.Write (LogLevel.Notice, "Registering application:");
+				Logger.Write(LogLevel.Notice, "    Host:          {0}", vhost ?? "any");
+				Logger.Write(LogLevel.Notice, "    Port:          {0}", (vport != -1) ? vport.ToString () : "any");
 
-				Console.WriteLine("    Virtual path:  {0}", vpath);
-				Console.WriteLine("    Physical path: {0}", fullPath);
+				Logger.Write(LogLevel.Notice, "    Virtual path:  {0}", vpath);
+				Logger.Write(LogLevel.Notice, "    Physical path: {0}", fullPath);
 			}
 
 			vpathToHost.Add (new VPathToHost (vhost, vport, vpath, fullPath));
@@ -173,14 +156,13 @@ namespace Mono.WebServer
 
  		public void AddApplicationsFromConfigDirectory (string directoryName)
  		{
-			if (verbose && !single_app) {
-				Console.WriteLine ("Adding applications from *.webapp files in " +
-						   "directory '{0}'", directoryName);
+			if (Verbose && !SingleApplication) {
+				Logger.Write(LogLevel.Notice, "Adding applications from *.webapp files in directory '{0}'", directoryName);
 			}
 
-			DirectoryInfo di = new DirectoryInfo (directoryName);
+			var di = new DirectoryInfo (directoryName);
 			if (!di.Exists) {
-				Console.Error.WriteLine ("Directory {0} does not exist.", directoryName);
+				Logger.Write (LogLevel.Error, "Directory {0} does not exist.", directoryName);
 				return;
 			}
 			
@@ -190,28 +172,26 @@ namespace Mono.WebServer
 
  		public void AddApplicationsFromConfigFile (string fileName)
  		{
-			if (verbose && !single_app) {
-				Console.WriteLine ("Adding applications from config file '{0}'", fileName);
+			if (Verbose && !SingleApplication) {
+				Logger.Write(LogLevel.Notice, "Adding applications from config file '{0}'", fileName);
 			}
 
 			try {
-				XmlDocument doc = new XmlDocument ();
+				var doc = new XmlDocument ();
 				doc.Load (fileName);
 
 				foreach (XmlElement el in doc.SelectNodes ("//web-application")) {
 					AddApplicationFromElement (el);
 				}
 			} catch {
-				Console.WriteLine ("Error loading '{0}'", fileName);
+				Logger.Write(LogLevel.Error, "Error loading '{0}'", fileName);
 				throw;
 			}
 		}
 
-		void AddApplicationFromElement (XmlElement el)
+		void AddApplicationFromElement (XmlNode el)
 		{
-			XmlNode n;
-
-			n = el.SelectSingleNode ("enabled");
+			XmlNode n = el.SelectSingleNode ("enabled");
 			if (n != null && n.InnerText.Trim () == "false")
 				return;
 
@@ -227,7 +207,7 @@ namespace Mono.WebServer
 			// TODO: support vhosts in xsp.exe
 			string name = el.SelectSingleNode ("name").InnerText;
 			if (verbose && !single_app)
-				Console.WriteLine ("Ignoring vhost {0} for {1}", n.InnerText, name);
+				Logger.Write (LogLevel.Warning, ("Ignoring vhost {0} for {1}", n.InnerText, name);
 #endif
 
 			int vport = -1;
@@ -238,7 +218,7 @@ namespace Mono.WebServer
 #else
 			// TODO: Listen on different ports
 			if (verbose && !single_app)
-				Console.WriteLine ("Ignoring vport {0} for {1}", n.InnerText, name);
+				Logger.Write (LogLevel.Warning, ("Ignoring vport {0} for {1}", n.InnerText, name);
 #endif
 
 			AddApplication (vhost, vport, vpath, path);
@@ -249,11 +229,11 @@ namespace Mono.WebServer
  			if (applications == null)
  				throw new ArgumentNullException ("applications");
  
- 			if (applications == "")
+			if (applications.Length == 0)
 				return;
 
-			if (verbose && !single_app) {
-				Console.WriteLine("Adding applications '{0}'...", applications);
+			if (Verbose && !SingleApplication) {
+				Logger.Write(LogLevel.Notice, "Adding applications '{0}'...", applications);
 			}
 
  			string [] apps = applications.Split (',');
@@ -266,16 +246,9 @@ namespace Mono.WebServer
 								     "[[hostname:]port:]VPath:realpath");
 
 				int vport;
-				string vhost;
-				string vpath;
-				string realpath;
 				int pos = 0;
 
-				if (app.Length >= 3) {
-					vhost = app[pos++];
-				} else {
-					vhost = null;
-				}
+				string vhost = app.Length >= 3 ? app[pos++] : null;
 
 				if (app.Length >= 4) {
 					// FIXME: support more than one listen port.
@@ -284,20 +257,20 @@ namespace Mono.WebServer
 					vport = -1;
 				}
 
-				vpath = app [pos++];
-				realpath = app[pos++];
+				string vpath = app [pos++];
+				string realpath = app[pos++];
 
 				if (!vpath.EndsWith ("/"))
 					vpath += "/";
  
- 				string fullPath = System.IO.Path.GetFullPath (realpath);
+ 				string fullPath = Path.GetFullPath (realpath);
 				AddApplication (vhost, vport, vpath, fullPath);
  			}
  		}
 
+		[Obsolete]
 		public bool Start (bool bgThread, Exception initialException, int backlog)
 		{
-			this.initialException = initialException;
 			return Start (bgThread, backlog);
 		}
 		
@@ -309,8 +282,8 @@ namespace Mono.WebServer
  			if (vpathToHost == null)
  				throw new InvalidOperationException ("SetApplications must be called first.");
 
-			if (single_app) {
-				VPathToHost v = (VPathToHost) vpathToHost [0];
+			if (SingleApplication) {
+				var v = vpathToHost [0];
 				v.AppHost = AppHost;
 				// Link the host in the application domain with a request broker in the *same* domain
 				// Not needed for SingleApplication and mod_mono
@@ -320,8 +293,7 @@ namespace Mono.WebServer
 
 			listen_socket = webSource.CreateSocket ();
 			listen_socket.Listen (backlog);
-			runner = new Thread (new ThreadStart (RunServer));
-			runner.IsBackground = bgThread;
+			runner = new Thread (RunServer) {IsBackground = bgThread};
 			runner.Start ();
 			stop = false;
 			return true;
@@ -339,7 +311,7 @@ namespace Mono.WebServer
 			webSource.Dispose ();
 
 			// A foreground thread is required to end cleanly
-			Thread stopThread = new Thread (new ThreadStart (RealStop));
+			var stopThread = new Thread (RealStop);
 			stopThread.Start ();
 		}
 
@@ -389,7 +361,7 @@ namespace Mono.WebServer
 		void RunServer ()
 		{
 			started = true;
-			SocketAsyncEventArgs args = new SocketAsyncEventArgs ();
+			var args = new SocketAsyncEventArgs ();
 			args.Completed += OnAccept;
 			listen_socket.AcceptAsync (args);
 			if (runner.IsBackground)
@@ -399,31 +371,30 @@ namespace Mono.WebServer
 				Thread.Sleep (1000000);
 		}
 
-		void OnAccept (object sender, EventArgs e)
+		void OnAccept (object sender, SocketAsyncEventArgs e)
 		{
-			if (!started && single_app) {
+			if (!started && SingleApplication) {
 				// We are shutting down. Last call...
 				Environment.Exit (0);
 			}
 
-			SocketAsyncEventArgs args = (SocketAsyncEventArgs) e;
-			Socket accepted = args.AcceptSocket;
-			args.AcceptSocket = null;
+			Socket accepted = e.AcceptSocket;
+			e.AcceptSocket = null;
 
-			if (args.SocketError != SocketError.Success) {
+			if (e.SocketError != SocketError.Success) {
 				CloseSocket(accepted);
 				accepted = null;
 			}
 
 			try {
 				if (started)
-					listen_socket.AcceptAsync (args);
+					listen_socket.AcceptAsync (e);
 			} catch (Exception ex) {
 				if (accepted != null)
 					CloseSocket (accepted);
 
 				// not much we can do. fast fail by killing the process.
-				Console.Error.WriteLine ("Unable to accept socket. Exiting the process. {0}", ex);
+				Logger.Write (LogLevel.Error, "Unable to accept socket. Exiting the process. {0}", ex);
 				Environment.Exit (1);
 			}
 
@@ -455,7 +426,7 @@ namespace Mono.WebServer
 
 		void SendException (Socket socket, Exception ex)
 		{
-			StringBuilder sb = new StringBuilder ();
+			var sb = new StringBuilder ();
 			string now = DateTime.Now.ToUniversalTime ().ToString ("r");
 
 			sb.Append ("HTTP/1.0 500 Server error\r\n");
@@ -474,34 +445,27 @@ namespace Mono.WebServer
 
 			byte[] data = Encoding.UTF8.GetBytes (sb.ToString ());
 			try {
-				socket.Send (data);
+				int sent = socket.Send (data);
+				if (sent != data.Length)
+					throw new IOException ("Blocking send did not send entire buffer");
 			} catch (Exception ex2) {
-				Console.WriteLine ("Failed to send exception:");
-				Console.WriteLine (ex);
-				Console.WriteLine ();
-				Console.WriteLine ("Exception ocurred while sending:");
-				Console.WriteLine (ex2);
+				Logger.Write(LogLevel.Error, "Failed to send exception:");
+				Logger.Write(ex);
+				Logger.Write(LogLevel.Error, "Exception ocurred while sending:");
+				Logger.Write(ex2);
 			}	
 		}
 		
 		void StartRequest (Socket accepted, int reuses)
 		{
-			Worker worker = null;
 			try {
-				if (initialException != null) {
-					SendException (accepted, initialException);
-					initialException = null;
-					accepted.Close ();
-					return;
-				}
-				
 				// The next line can throw (reusing and the client closed)
-				worker = webSource.CreateWorker (accepted, this);
+				Worker worker = webSource.CreateWorker (accepted, this);
 				worker.SetReuseCount (reuses);
-				if (false == worker.IsAsync)
-					ThreadPool.QueueUserWorkItem (new WaitCallback (worker.Run));
-				else
+				if (worker.IsAsync)
 					worker.Run (null);
+				else
+					ThreadPool.QueueUserWorkItem (worker.Run);
 			} catch (Exception) {
 				try {
 					if (accepted != null) {
@@ -528,14 +492,14 @@ namespace Mono.WebServer
 		public VPathToHost GetApplicationForPath (string vhost, int port, string path,
 							  bool defaultToRoot)
 		{
-			if (single_app)
-				return (VPathToHost) vpathToHost [0];
+			if (SingleApplication)
+				return vpathToHost [0];
 
 			VPathToHost bestMatch = null;
 			int bestMatchLength = 0;
 
 			for (int i = vpathToHost.Count - 1; i >= 0; i--) {
-				VPathToHost v = (VPathToHost) vpathToHost [i];
+				var v = vpathToHost [i];
 				int matchLength = v.vpath.Length;
 				if (matchLength <= bestMatchLength || !v.Match (vhost, port, path))
 					continue;
@@ -555,8 +519,8 @@ namespace Mono.WebServer
 			if (defaultToRoot)
 				return GetApplicationForPath (vhost, port, "/", false);
 
-			if (verbose)
-				Console.WriteLine ("No application defined for: {0}:{1}{2}", vhost, port, path);
+			if (Verbose)
+				Logger.Write(LogLevel.Error, "No application defined for: {0}:{1}{2}", vhost, port, path);
 
 			return null;
 		}
@@ -564,7 +528,7 @@ namespace Mono.WebServer
 		public VPathToHost GetSingleApp ()
 		{
 			if (vpathToHost.Count == 1)
-				return (VPathToHost) vpathToHost [0];
+				return vpathToHost [0];
 			return null;
 		}
 
@@ -572,7 +536,7 @@ namespace Mono.WebServer
 		{
 			// Called when the host appdomain is being unloaded
 			for (int i = vpathToHost.Count - 1; i >= 0; i--) {
-				VPathToHost v = (VPathToHost) vpathToHost [i];
+				var v = vpathToHost [i];
 				if (v.TryClearHost (host))
 					break;
 			}
